% trainContextModel.m
%
% Trains the top-down contextual guidance model described in Torralba et
% al. (2006); Ehinger et al. (2009).  This code uses the labelMe database
% for training images and requires the labelMe toolbox, which can be
% downloaded from:
%
% http://labelme.csail.mit.edu/

clear all
addpath('features');


%% PARAMETERS
% Target object (see lableMe toolbox documentation for query format)
%objectTarget = 'person+walking';
objectTarget = 'car';

% Training parameters
Nrepetitions = 20; % each image is cropped randomly N times
perc=.9; % percentage of data that will be used for training
parameterFile = strrep(objectTarget, ',', '');
parameterFile = strrep(parameterFile, '+', '');
parameterFile = strrep(parameterFile, '-', '');

% Global features parameters
Nsc = 4; Nor = 6; Nb = 4;
desiredHeight = 256;
desiredWidth = 256;

% Warn if the chosen parameters will overwrite an existing parameter file
if exist(['contextModels/' parameterFile '.mat'],'dir') ~= 0
    button = questdlg(['contextModels/' parameterFile '.mat already exists; okay to overwrite?'],'Warning: File overwrite','Yes','No','Yes');
    if strcmp(button,'No')
        return
    end
end

%% PRODUCE TRAINING DATA using LabelMe dataset
LabelMeHOMEANNOTATIONS = 'http://labelme.csail.mit.edu/Annotations';
LabelMeHOMEIMAGES = 'http://labelme.csail.mit.edu/Images';
database = LMdatabase(LabelMeHOMEANNOTATIONS,LabelMeHOMEIMAGES);


%% COMPUTE FEATURES for test and training data
[Dc, jc]  = LMquery(database, 'folder', '256x256');
[Dc, jc]  = LMquery(database, 'folder', '-seq');
[Dc, jc]  = LMquery(database, 'object.name', objectTarget);
[Dc, jc]  = LMquery(Dc, 'object.name', objectTarget);
Nimages = length(Dc)
precomputeFeatures


%% TRAIN WITH MIXTURE GAUSSIANS
% PCA
npcaWFT=100;
[featPC, pc]=pca(data.featuresPositive, npcaWFT);

% split the test and learning groups:
Nimages = max(data.imagendx);
r = randperm(Nimages);
Learn = r(1:fix(Nimages*perc)); learn = find(ismember(data.imagendx, Learn));
Test = r(1+fix(Nimages*perc):Nimages); test = find(ismember(data.imagendx, Test));
vL=featPC(:,learn);
xL = data.x(learn); yL = data.y(learn);
vT=featPC(:,test);
xT = data.x(test); yT = data.y(test);

% Fit mixture of gaussians with local regresion
NgF=4; 
SigmaX=000.4; 
iterF=145; 
np = 32;

[fv,pF,mxF,CxF,CyF,bF]=CWMFocus([xL*0.01+mean(xL); yL], vL(1:np,:), NgF, iterF, 1, SigmaX);

% save parameters
save(['contextModels/' parameterFile], 'pF', 'mxF', 'CxF', 'CyF', 'bF', 'np', 'pc', 'desiredHeight', 'desiredWidth', 'Nsc', 'Nor', 'Nb');

% save all temporary data
save(['tmp/' parameterFile])


%% TEST
% p(x | on, v)
ux=-30:8:(desiredHeight+30);
[xx, yy]=meshgrid(ux,ux);
Sxy=[xx(:) yy(:)]';
disp('TEST mixture gaussians...')

clear mXY
for k = 1:length(test)
    fdtF = FyCWM(Sxy,vT(1:np,k),pF, mxF,CxF,CyF, bF);
    fdtF = fdtF / sum(fdtF);
    mXY(k, :) = sum(Sxy'.*repmat(fdtF,1,2));
end

figure
plot(yT,mXY(:,2),'.')
%axis([1 128 1 128]); 
axis('square'); grid
c = (corrcoef(yT, mXY(:,2)));
title(c(1,2))

% Show images on test set
figure
objectSize = [16 16];
%objectSize = [32 32];
for n = 1:length(Test)
    img = LMimread(Dc, Test(n), LabelMeHOMEIMAGES);
    img = uint8(mean(img,3));
    
    [newannotation, newimg, crop, scaling, err, msg] = LMcookimage(Dc(Test(n)).annotation, img, ...
        'objectname', objectTarget, 'objectsize', objectSize, 'objectlocation', 'random', ...
        'maximagesize', [desiredHeight desiredWidth]);

    if size(newimg,1) < desiredHeight | size(newimg,2)<desiredWidth; err=1; end

    % Only use for test images with the right image size
    if err==0
        focusOfAttention = foa(newimg, ['contextModels\' parameterFile]);

        subplot(121)
        imshow(newimg)
        subplot(122)
        imshow(uint8(double(newimg).*focusOfAttention))
        
        drawnow
    end
end



